import pandas as pd
import json

def read_tweets(tweet_dir):
    tweets = []

    for lookup_file in tweet_dir:
        with open(lookup_file, encoding="utf-8") as f:
            for line in f:
                tweet = json.loads(line)
                d = {
                    "id": tweet["id"],
                    "text": tweet["text"],
                    "created_at": tweet["created_at"],
                    "user_name": tweet["user"]["screen_name"],
                    "id_str": tweet["id_str"],
                    "urls": tweet["entities"]["urls"]
                }
                tweets.append(d)

    concatenated_df = pd.DataFrame(tweets)
    concatenated_df = _specify_tweets_with_urls(concatenated_df)
    
    return concatenated_df

def wide_to_long_url (df_tweets):
    long_urls_list = []

    for index, tweet in df_tweets.iterrows():
        for url in tweet['urls']:
            tweet_url = {
                'id': tweet['id'],
                'created_at': tweet['created_at'],
                'url': url['url'],
                'expanded_url': url['expanded_url'],
                'user_name': tweet['user_name']
            }
            long_urls_list.append(tweet_url)

    df_long_urls = pd.DataFrame(long_urls_list)
    return df_long_urls

def _specify_tweets_with_urls (df):
    df['urls'] = df['urls'].apply(lambda x: list(x))
    df['has_url'] = df['urls'].apply(lambda x: 'yes' if len(x) > 0 else 'no')
    return df




#!/usr/bin/env python
# encoding: utf-8

"""CLI for Domain_Network project."""


from domainNetwork.domain_network_utils import *
from domainNetwork.tweet_reader import *
from domainNetwork.url_processor import *
from domainNetwork.network_generator import *

import argparse
import configparser
import glob



def parse_arguments():

    # parse arguments if available
    parser = argparse.ArgumentParser(
        description="Domain network"
    )

    # File path to the data.
    parser.add_argument(
        "--input_dir",
        type=str,
        help="File path to the dataset of tweets"
    )

    # File path to the config file.
    parser.add_argument(
        "--conf_dir",
        type=str,
        help="File path to the config file"
    )

    parser.add_argument(
        "--min_edge_weight",
        type=int,
        default=1,
        help="Minimum number of users that mention two netlocs together, to be shown in the network "
    )
    parser.add_argument(
        "--min_node_size",
        type=int,
        default=1,
        help="Minimum number of times a netloc is mentioned in total, to be shown in the network")
    parser.add_argument(
        "--min_stand_alone_size",
        default=1,
        type=int,
        help="Minimum number of times a stand-alone netloc is mentioned in total, to be shown in the network")

    #File pathes to output
    parser.add_argument(
        "--network_only",
        type=bool,
        default=False,
        help="Read preprocessed file and make a network, instead of reading row tweet files and preprocess it"
    )
    parser.add_argument(
        "--urls_file_name",
        type=str,
        default=None,
        help="File path to save urls including redirected ones"
    )

    parser.add_argument(
        "--network_output_file_name",
        type=str,
        default=None,
        help="File path to save network"
    )
    parser.add_argument(
        "--netloc_output_file_name",
        type=str,
        default=None,
        help="File path to save list of netlocs"
    )

    parser.add_argument(
        "--netloc_origin_output_file_name",
        type=str,
        default=None,
        help="File path to save original list of netlocs"
    )
    return parser


def main():
    parser = parse_arguments()
    args = parser.parse_args()

    # Read config file
    config = configparser.ConfigParser()
    config.read(args.conf_dir) # 'config/config.ini'

    print(config.sections())

    url_shorteners = config['FILTERING']['url_shortner_list']
    url_shortener_list =url_shorteners.strip('[]').split(',')

    nodes_to_remove  = config['FILTERING']['nodes_to_remove']
    nodes_list_to_remove =nodes_to_remove.strip('[]').split(',')

    dic = config['FILTERING']['replace_netlocs']
    dict_change_netlocs = json.loads(dic)


    if args.network_only:
        #Read processed url file
        df_urls = read_dataframe(args.urls_file_name)
        print("df_urls: ",df_urls.shape)

    else:
        #Read tweets and preprocess it
        path = args.input_dir
        all_files = glob.glob(path)
        df_tweets_origin = read_tweets(all_files)
        df_tweets = wide_to_long_url(df_tweets_origin)

        #Process URLs
        df_urls = process_urls(df_tweets, url_shortener_list, args.urls_file_name)

    domain_network = DomainNetwork(
        df_urls,
        nodes_list_to_remove,
        args.min_edge_weight,
        args.min_node_size,
        args.min_stand_alone_size,
        args.network_output_file_name,
        args.netloc_output_file_name,
        args.netloc_origin_output_file_name
    )

    domain_network.combine_nodes(dict_change_netlocs)
    domain_network.trim_nodes()

    domain_network.create_domain_network()
    domain_network.replace_nodes_in_network(dict_change_netlocs)
    domain_network.trim_domain()
    domain_network.add_stand_alone_node_to_network()
    domain_network.save_network()
    

# execute main function
if __name__ == "__main__":
    main()


import re
import requests
import os
from domainNetwork.domain_network_utils import save_dataframe
from urllib.parse import urlsplit


def _redirect_link(url):
    try:
        r = requests.head(url, timeout=1, allow_redirects=True)
        print(r.url)
        return r.url
    except Exception:
        return url


def _clean_netloc(text):
    text = re.sub(r"www.", "", text, flags=re.MULTILINE)
    return text

def _clean_netlocs(df):
    """ add a column with the clean_netloc, 'www.' is removed, to the data """
    df['url_netloc_clean'] = df['url_netloc'].apply(_clean_netloc)
    return df


def _get_redirect_links(df_urls, shortened_urls_to_redirect):
    """ update column expanded_url with the redirect link
       Takes too much time, if we want to apply it for all
    """
    df_urls['expanded_url_redirected'] = df_urls.loc[df_urls['url_netloc_clean'].isin(shortened_urls_to_redirect),\
                      'expanded_url'].apply(_redirect_link)

    df_urls.loc[~df_urls['expanded_url_redirected'].isnull(),'expanded_url'] = \
            df_urls.loc[~df_urls['expanded_url_redirected'].isnull(),'expanded_url_redirected']
    return df_urls

def _extract_netloc(df):
    ''' add a column with the netloc to the data'''
    df['url_netloc'] = df['expanded_url'].apply(lambda x: urlsplit(x).netloc)
    return df


def process_urls(df_urls, url_shortener_list, processed_output_file):

    df_urls = _extract_netloc(df_urls)

    df_urls = _clean_netlocs(df_urls)

    df_urls = _get_redirect_links(df_urls, url_shortener_list)

    """repeat for new redirected links"""
    df_urls = _extract_netloc(df_urls)

    """repeat for new redirected links"""
    df_urls = _clean_netlocs(df_urls)

    save_dataframe(df_urls, processed_output_file)

    return df_urls


import pandas as pd
import itertools
from domainNetwork.domain_network_utils import save_dataframe


class DomainNetwork:

    def __init__(self,
                 df,
                 nodes_list_to_remove,
                 min_edge_weight,
                 min_node_size,
                 min_stand_alone_size,
                 network_output_file_name,
                 netloc_output_file_name,
                 netloc_origin_output_file_name
                 ):
        self.df = df
        self.nodes_list_to_remove = nodes_list_to_remove
        self.min_edge_weight = min_edge_weight
        self.min_node_size = min_node_size
        self.min_stand_alone_size = min_stand_alone_size

        self.user_names = self.df.user_name.unique()

        """a dataframe of netlocs and the total number of times a netloc is mentioned """
        self.df_netlocs = pd.DataFrame(self.df[['url_netloc_clean']].groupby(['url_netloc_clean']).size().reset_index(name='count'))
        self.df_netlocs_origin = pd.DataFrame(self.df[['url_netloc_clean']].groupby(['url_netloc_clean']).size().reset_index(name='count'))

        print("self.df", self.df.shape)
        print("self.df_netlocs in init",self.df_netlocs.shape)
        self.df_network = None

        self.network_output_file_name = network_output_file_name
        self.netloc_output_file_name = netloc_output_file_name
        self.netloc_origin_output_file_name = netloc_origin_output_file_name


    def get_network_per_user(self, df_user_netloc, username):
        '''select netlocs mentioned by a specific user'''
        ntlc = list(df_user_netloc.loc[df_user_netloc['user_name'] == username, 'url_netloc_clean'])

        '''select all combination of two netlocs'''
        sub_2 = [subset for subset in itertools.combinations(ntlc, 2)]
        return sub_2

    def create_domain_network(self):
        """ create a groupby dataframe of users, netlocs and the count """
        df_user_netloc_count = (self.df.groupby(['user_name', 'url_netloc_clean']).size().reset_index(name='count'))

        """create a list of source target for all users"""
        netloc_s_t = sum((self.get_network_per_user(df_user_netloc_count, usr) for usr in self.user_names), [])

        """aggregate list of source targets and give the weights"""
        netloc_s_t = pd.DataFrame(netloc_s_t, columns=['source', 'target'])
        self.df_network = (netloc_s_t.groupby(['source', 'target']).size().reset_index(name='weight'))

        """Since groupby to create df_user_netloc_count, is alphabetically ordered by default, 
            it seems we do not endup with rows that source and target values are switched"""


    def combine_2nodes(self, node_to_replace, node_to_remove):
        replace_node_count = self.df_netlocs.loc[self.df_netlocs['url_netloc_clean'] == node_to_replace, 'count']
        remove_node_count = self.df_netlocs.loc[self.df_netlocs['url_netloc_clean'] == node_to_remove, 'count']

        replace_node_count_val = replace_node_count.values[0] if len(replace_node_count) > 0 else 0
        remove_node_count_val = remove_node_count.values[0] if len(remove_node_count) > 0 else 0

        self.df_netlocs.loc[self.df_netlocs['url_netloc_clean'] == node_to_replace, 'count'] = replace_node_count_val + remove_node_count_val
        self.df_netlocs = self.df_netlocs[self.df_netlocs.url_netloc_clean != node_to_remove]

    def combine_nodes(self, dict_change_netlocs):
        for key, value in dict_change_netlocs.items():
            node_to_replace = key
            node_to_remove = dict_change_netlocs[key]

            self. combine_2nodes(node_to_replace, node_to_remove)

        print("df_netlocs",self.df_netlocs.shape)

    def filter_by_node_size(self):
        count_filter = self.df_netlocs['count'] >= self.min_node_size
        self.df_netlocs = self.df_netlocs.loc[count_filter]

    def filter_nodes_to_remove(self):
        removing_netlocs_filter = self.df_netlocs['url_netloc_clean'].isin(self.nodes_list_to_remove)
        self.df_netlocs = self.df_netlocs.loc[~removing_netlocs_filter]

    def trim_nodes(self):
        self.filter_by_node_size()
        print("df_netlocs after filter by node size", self.df_netlocs.shape)
        self.filter_nodes_to_remove()
        print("self.df_netlocs after removing social .. ",self.df_netlocs.shape)

    def swap_source_target(self, cond):
        compare_filter = self.df_network['source'] > self.df_network['target']
        self.df_network.loc[compare_filter & cond, ['source', 'target']] = (
            self.df_network.loc[compare_filter & cond, ['target', 'source']].values)


    def replace_node_in_network(self, node_to_replace, node_to_remove):
        """Replace node_to_remove with node_to_replace in the network"""
        self.df_network.loc[self.df_network['source'] == node_to_remove, 'source'] = node_to_replace
        self.df_network.loc[self.df_network['target'] == node_to_remove, 'target'] = node_to_replace

        """Swap source and target if target < source alphabetically"""
        cond = (self.df_network['source'] == node_to_replace) | (self.df_network['target'] == node_to_replace)
        self.swap_source_target(cond)
        self.df_network = pd.DataFrame(self.df_network.groupby(['source', 'target']).sum()['weight']).reset_index()

    def replace_nodes_in_network(self, dict_change_netlocs):
        for key, value in dict_change_netlocs.items():
            node_to_replace = key
            node_to_remove = dict_change_netlocs[key]
            self. replace_node_in_network(node_to_replace, node_to_remove)

    def remove_edge_by_weight(self):
        weight_filter = self.df_network['weight'] >= self.min_edge_weight
        self.df_network = self.df_network[weight_filter]

    def remove_network_not_found_in_nodeList(self):
        """remove network of nodes that are not in node list.
        Including nodes< node_size and nodes in removing list """

        desired_node_list = list(self.df_netlocs['url_netloc_clean'])
        source_filter = self.df_network['source'].isin(desired_node_list)
        target_filter = self.df_network['target'].isin(desired_node_list)
        self.df_network = self.df_network[source_filter & target_filter]

    def trim_domain(self):
        self.remove_edge_by_weight()
        self.remove_network_not_found_in_nodeList()

    def find_stand_alone_nodes(self):
        source_filter = self.df_netlocs['url_netloc_clean'].isin(self.df_network['source'])
        target_filter = self.df_netlocs['url_netloc_clean'].isin(self.df_network['target'])
        nodesize_filter = self.df_netlocs['count'] >= self.min_stand_alone_size

        stand_alone_netlocs = list(self.df_netlocs.loc[~source_filter & ~target_filter,'url_netloc_clean'])
        popular_stand_alone_netlocs = list(self.df_netlocs.loc[~source_filter & ~target_filter & nodesize_filter, 'url_netloc_clean'])
        return stand_alone_netlocs,popular_stand_alone_netlocs


    def add_stand_alone_node_to_network(self):
        """add stand-alone nodes to the network and give weight=1000"""
        stand_alone_netlocs, popular_stand_alone_netlocs = self.find_stand_alone_nodes()

        """add popular stand-alones to the network """
        standalone_rows = [[u,u,1000] for u in popular_stand_alone_netlocs]
        standalone_df = pd.DataFrame(standalone_rows, columns=['source','target','weight'])
        self.df_network = self.df_network.append(standalone_df, ignore_index=True)

        """remove non-popular standalones from node list"""
        non_popular_standalones = [np for np in stand_alone_netlocs if np not in popular_stand_alone_netlocs]
        non_popular_filter = self.df_netlocs['url_netloc_clean'].isin(non_popular_standalones)
        self.df_netlocs = self.df_netlocs[~non_popular_filter]

    def save_network(self):

        save_dataframe(self.df_netlocs, self.netloc_output_file_name)
        save_dataframe(self.df_network, self.network_output_file_name)
        save_dataframe(self.df_netlocs_origin, self.netloc_origin_output_file_name)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="domain_network",
    version="0.0.1",
    author="Research-IT support",
    author_email="p.zahedi@uu.nl",
    description="Makes a network out of a URLs in a dataset of tweets",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.science.uu.nl/research-it-support/domain_network",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
install_requires=[
        'numpy',
        'pandas'
    ]
)